@@ -5,7 +5,6 @@ from typing import Any, Dict, List, Optional, Union
 
 import opik.api_objects.attachment as attachment
 import opik.datetime_helpers as datetime_helpers
-import opik.dict_utils as dict_utils
 import opik.llm_usage as llm_usage
 from opik.types import (
     ErrorInfoDict,
@@ -13,8 +12,7 @@ from opik.types import (
     LLMProvider,
     SpanType,
 )
-
-from .. import helpers
+from .. import helpers, data_helpers
 
 LOGGER = logging.getLogger(__name__)
 
@@ -109,40 +107,27 @@ class SpanData:
                 continue
 
             if key == "metadata":
-                self._update_metadata(value)
+                self.metadata = data_helpers.merge_metadata(
+                    self.metadata, new_metadata=value
+                )
                 continue
             elif key == "output":
-                self._update_output(value)
+                self.output = data_helpers.merge_outputs(self.output, new_outputs=value)
                 continue
             elif key == "input":
-                self._update_input(value)
+                self.input = data_helpers.merge_inputs(self.input, new_inputs=value)
                 continue
             elif key == "attachments":
                 self._update_attachments(value)
                 continue
+            elif key == "tags":
+                self.tags = data_helpers.merge_tags(self.tags, new_tags=value)
+                continue
 
             self.__dict__[key] = value
 
         return self
 
-    def _update_metadata(self, new_metadata: Dict[str, Any]) -> None:
-        if self.metadata is None:
-            self.metadata = new_metadata
-        else:
-            self.metadata = dict_utils.deepmerge(self.metadata, new_metadata)
-
-    def _update_output(self, new_output: Dict[str, Any]) -> None:
-        if self.output is None:
-            self.output = new_output
-        else:
-            self.output = dict_utils.deepmerge(self.output, new_output)
-
-    def _update_input(self, new_input: Dict[str, Any]) -> None:
-        if self.input is None:
-            self.input = new_input
-        else:
-            self.input = dict_utils.deepmerge(self.input, new_input)
-
     def init_end_time(self) -> "SpanData":
         self.end_time = datetime_helpers.local_timestamp()
 
